﻿/*
Select By Material version history
--============================================================
Version 1.0 - 2015.07.24
- First Release. (Select your objects quickly + auto-open/close group heads)

Version 1.1 - 2015.07.25
- ADDED: Support for groups that were previously opened while still closing the groups affected by the script.
- ADDED: Object count in listener.
- ADDED: Support for Compact Material Editor, you can now toggle back and front between them with no problem.
- ADDED: Warning that no object/s with selected material exist in scene.
- CHG: Script name to 'Select By Material'.
- ADDED: Querybox to open material editor if not opened.

Version 1.2 - 2015.08.29
- CHG: Supports max 2014 only, this is due to 2014s getSelectedNodes() array in max.
- CHG: Output results of the no. of items selected and materials names in a messagebox instead of listener. Listener if only one material node is selected.
- ADDED: Multiple selection of material nodes in SME now supported like normal method, only much quicker.
- ADDED: Support for Environment Maps where added to the environment slot.
- ADDED: Zoom to selection.
- REMOVED: Selection of sub materials in Multi-Sub objects after changing to selection of nodes.
- BUG FIX: Works with spaces in SME views name.
--============================================================
Version 1.3 - 2016.11.15
- ADDED: Unhide objects and hide again on scripts second run.
--============================================================
Version 1.4 - 2016.11.27
- BUG FIX: SME select function now works on objects as opposed to geometry only.
- BUG FIX: Made sure o.parent is a valid Grouphead prior to collection.
- BUG FIX: Deleted scene objects are removed from arrays.
--============================================================

NOTES:
--============================================================
- Works with groups, nested groups and closes any groups the script has affected while keeping your prevoius groups opened.
- Does the same with hidden objects also.

TO DO LIST
--============================================================
- TBC
-- Please comment on my scriptspot page if you have any suggestions: http://www.scriptspot.com/3ds-max/scripts/select-by-material
--============================================================ */

	if MatEditor.isOpen() == false then
	(
		if querybox "Material Editor not open !\n\n Open now and select node/s OR slot, then re-run script." title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true then
		(
			MatEditor.Open()
		)
	)
	else
	(

		if objects.count > 0 then
		(

			global groupclosedcopy_arr
			global hiddenobjectcopy_arr
			local placestocheckmsg = "\n\nAlternatively check:\n\n»    If material is part of a sub-material.\n»    Environment slot/s inside your render engine settings.\n»    Material override slot inside your render engine settings."
			local matSelect_arr
			global selNodeNames_arr

			if MatEditor.isOpen() == true AND maxOps.mtlDlgMode == #advanced AND sme.activeView != 0 then
			(
				v = sme.activeView
				view = sme.GetView v
				numNodes = view.GetNumNodes()

				selNodeInterface = view.getSelectedNodes() --get selected node <MixinInterface:Node>

				viewMatNames = 	for i = 1 to trackViewNodes[#sme][sme.activeView].numsubs where isProperty trackViewNodes[#sme][sme.activeView][i] "reference" AND superclassof trackViewNodes[#sme][sme.activeView][i].reference == material collect try(trackViewNodes[#sme][sme.activeView][i].reference.name)catch()
				--viewMatNames = for n = 1 to numNodes where (superclassof trackViewNodes[#sme][v][n].reference == material) collect trackViewNodes[#sme][v][n].reference.name
				selNodeNames_arr = sort(for m in selNodeInterface collect m.name) --collect all selected node/s names
				selNodeMatsInSceneNames = sort(for o in sceneMaterials where o.name != undefined AND FindItem selNodeNames_arr o.name > 0 collect o.name) --collect only the names of materials in the scene

				--copy across arrays
				viewMatNames_1 = copy viewMatNames #nomap
				selNodeMapNames = copy selNodeNames_arr #nomap
				selNodeMatNames = copy selNodeNames_arr #nomap

				--gets selected non material nodes only
				for i in viewMatNames_1 do
				(
					id = finditem selNodeMapNames i if id != 0 then
					(
						deleteItem selNodeMapNames id
					)
				)
				sort selNodeMapNames

				--gets selected material nodes only
				for i in selNodeMapNames do
				(
					id = finditem selNodeMatNames i if id != 0 then
					(
						deleteItem selNodeMatNames id
					)
				)
				sort selNodeMatNames
				theNonSceneMat = copy selNodeMatNames #nomap

				--the non scene material nodes
				for i in selNodeMatsInSceneNames do
				(
					id = finditem theNonSceneMat i if id != 0 then
					(
						deleteItem theNonSceneMat id
					)
				)
				sort theNonSceneMat

				msgNonSceneMat	= ""
				for m in theNonSceneMat do msgNonSceneMat = msgNonSceneMat + ("»    " + m + "\n") as string --create messagebox with selected materials
				msgboxMats = ""
				for m in selNodeMatsInSceneNames do msgboxMats = msgboxMats + ("»    " + m + "\n") as string --create messagebox with selected materials
				msgboxMapsAll = ""
				for m in selNodeNames_arr do msgboxMapsAll = msgboxMapsAll + ("»    " + m + "\n") as string --create messagebox with materials and map names
			)

			fn clearDeadNodes deadArrRemove =
			(
				if deadArrRemove != undefined AND deadArrRemove.count != 0 do (for i = deadArrRemove.count to 1 by -1 where not isValidNode deadArrRemove[i] do deleteItem deadArrRemove i)
				deadArrRemove
			)

			fn groupCloseHide =
			(
				clearDeadNodes groupclosedcopy_arr
				clearDeadNodes hiddenobjectcopy_arr
				/* Groups */
				if groupclosedcopy_arr != undefined then
				(
					while groupclosedcopy_arr.count > 0 do
					(
						setGroupOpen groupclosedcopy_arr[groupclosedcopy_arr.count] false
						groupclosedcopy_arr.count = groupclosedcopy_arr.count-1
					)
				)
				/* Hidden Objs */
				if hiddenobjectcopy_arr != undefined then
				(
					while hiddenobjectcopy_arr.count > 0 do
					(
						hide hiddenobjectcopy_arr[hiddenobjectcopy_arr.count]
						hiddenobjectcopy_arr.count = hiddenobjectcopy_arr.count-1
					)
				)
			)

			fn groupOpenUnhide =
			(
				clearDeadNodes groupclosed_arr
				clearDeadNodes hiddenobject_arr
				/* Groups */
				groupclosed_arr = for o in matSelect_arr where isGroupMember o AND isOpenGroupMember o == false AND isGroupHead o.parent collect o.parent
				groupclosedcopy_arr = copy groupclosed_arr #nomap
				while groupclosed_arr.count > 0 do
				(
					setGroupOpen groupclosed_arr[groupclosed_arr.count] true
					groupclosed_arr.count = groupclosed_arr.count-1
				)
				/* Hidden Objs */
				hiddenobject_arr = for o in matSelect_arr where o.isHidden collect o
				hiddenobjectcopy_arr = copy hiddenobject_arr #nomap
				while hiddenobject_arr.count > 0 do
				(
					unhide hiddenobject_arr[hiddenobject_arr.count]
					hiddenobject_arr.count = hiddenobject_arr.count-1
				)
			)

			fn environmentmapcheckSME =
			(
				if environmentMap != undefined do if (FindItem selNodeNames_arr environmentMap.name > 0) then
				(
					o = true
				)
			)

			fn environmentmapcheckCME =
			(
				if environmentMap != undefined do if (meditMaterials [activeMeditSlot] == environmentMap) then
				(
					o = true
				)
			)

--============================================================
			if maxOps.mtlDlgMode == #basic then
			(
				clearselection()
				matSelect_arr = for o in objects where o.material == meditMaterials [activeMeditSlot] collect o
				groupCloseHide()
				groupOpenUnhide()
				select matSelect_arr
				CMEmatname = meditMaterials [activeMeditSlot].name
				matSelMulti_arr = for o in matSelect_arr where (classOf o.material == Multimaterial) collect o
				if matSelect_arr.count > 0 then
				(
					select matSelect_arr ; max zoomext sel
					print ((selection.count as string) + " Object/s selected with '" + CMEmatname + "' material")
				)
				if matSelect_arr.count == 0 AND environmentmapcheckCME() == undefined then
				(
					messagebox ("Compact Material Editor ACTIVE !\n\nNo object/s in the scene assigned the material:\n\n'" + CMEmatname + "'" + placestocheckmsg) title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
					print "No object/s selected !"
				)
				if matSelect_arr.count == 0 AND environmentmapcheckCME() != undefined then
				(
					messagebox "This map is part of the environment slot and is not applied to any object/s in the scene." title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
					print "No object/s selected !"
				)
			) -- end if
			else -- SME Select
			(
				if selNodeInterface.count > 0 do
				(
					matSelect_arr = for o in objects where o.material != undefined AND FindItem selNodeNames_arr o.material.name > 0 collect o
					setWaitCursor()
					clearselection()
					if matSelect_arr != undefined then
					(
						groupCloseHide()
						groupOpenUnhide()
					)

					-- environment map check
					if environmentmapcheckSME() == true AND selNodeInterface.count == 1 then
					(
						messagebox "The node you've selected is part of the main environment slot and is not applied to any object/s in the scene." title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
						if matSelect_arr.count == 0 then print "No object/s selected !"
					)
					if environmentmapcheckSME() == true AND matSelect_arr.count > 0 then
					(
						messagebox "A node you've selected is part of the main environment slot and is not applied to any object/s in the scene." title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
						if matSelect_arr.count == 0 then print "No object/s selected !"
					)

					-- selection of  objects if only 1 material
					if matSelect_arr.count > 0 AND theNonSceneMat.count == 0 AND selNodeMatsInSceneNames.count == 1 then
					(
						select matSelect_arr ; max zoomext sel
						print ((selection.count as string) + " Object/s selected with the following material: '" + selection[1].material.name + "' ") --title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
					)

					-- selection of  objects if more than 1 material
					if matSelect_arr.count > 0 AND theNonSceneMat.count == 0 AND selNodeMatsInSceneNames.count > 1 then
					(
						select matSelect_arr ; max zoomext sel
						messagebox ((selection.count as string) + " Object/s selected with the following " + selNodeMatsInSceneNames.count as string + " material/s:\n\n" + msgboxMats) title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
					)

					-- selection of  objects if missing materials selected
					if matSelect_arr.count > 0 AND theNonSceneMat.count > 0 then
					(
						select matSelect_arr ; max zoomext sel
						messagebox ((selection.count as string) + " Object/s selected with the following " + selNodeMatsInSceneNames.count as string + " material/s:\n\n" + msgboxMats + "\n\nMaterial/s not in the scene:\n\n" + msgNonSceneMat as string) title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
					)

					-- if no selected nodes are contained in the scene
					if matSelect_arr.count == 0 AND theNonSceneMat.count > 0 AND environmentmapcheckSME() == undefined then
					(
						messagebox ("Slate Material Editor ACTIVE !\n\nNo object/s in the scene assigned the material/s or map/s:\n\n" + msgNonSceneMat as string + placestocheckmsg) title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
						if matSelect_arr.count == 0 then print "No object/s selected !"
					)

					-- if only selected nodes are maps, controllers etc
					if matSelect_arr.count == 0 AND theNonSceneMat.count == 0 AND selNodeMapNames.count > 0 AND environmentmapcheckSME() == undefined then
					(
						messagebox ("Only node/s selected, please select material node/s and try again...") title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
						if matSelect_arr.count == 0 then print "No object/s selected !"
					)

				)
				-- no node/s selected warning
				if selNodeInterface.count == 0 do
				(
					clearselection()
					if matSelect_arr != undefined then
					(
						groupCloseHide()
						groupOpenUnhide()
					)

					if querybox "There's no node/s selected in the SME !\n\nDo you want to select the node/s now and re-run script ?\n\nN.B. SME will auto-open if YES is clicked." title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true then
					(
						groupCloseHide()
						print "Select node/s in SME first"
						SME.Open()
					)
					else
					(
						groupCloseHide()
					)
				)
				setArrowCursor()
			) -- end else
		)
		else messagebox "No objects in scene !\n\n\n#fail #tryharder #3dwannab #ilovemax" title:"SelectByMaterial_by_3dwannab (◣_◢)" beep:true
	) -- endscript
--============================================================ THATS ALL FOLKS !
